-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 4528 of the CMF schema.
-- Tighten role config group name uniqueness constraint
--

update SCHEMA_VERSION set VERSION=4528;

drop index IDX_UNIQUE_ROLE_CONFIG_GROUP;

-- At this point we only expect to see base groups, but we have to make
-- their names globally unique.
--
-- Change them from '<ROLE_TYPE>_BASE' to '<SERVICE_NAME>-<ROLE_TYPE>-BASE'.
update ROLE_CONFIG_GROUPS
  set NAME=(
      (select NAME
        from SERVICES left join CONFIG_REVISIONS
        on SERVICES.SERVICE_ID=CONFIG_REVISIONS.SERVICE_ID
        where ROLE_CONFIG_GROUPS.REVISION_ID=CONFIG_REVISIONS.REVISION_ID)
      || '-'
      || ROLE_CONFIG_GROUPS.ROLE_TYPE
      || '-BASE')
  where NAME like '%_BASE';

-- Sadly, we can't make them globally unique, at least not while revision
-- history is tangled up in our model. But we'll approximate it and make
-- the names future proof.
create unique index IDX_UNIQUE_ROLE_CONFIG_GROUP on ROLE_CONFIG_GROUPS
  (NAME, REVISION_ID);
